#pragma once
#include <iostream>

template <typename F, typename S>
class PairArray
{
public:
	// Default constructor
	PairArray();

	// Copy Constructor
	PairArray(const PairArray<F, S>& orig);

	void print(std::ostream& out) const;

	// Equal operator
	PairArray<F, S>& operator=(const PairArray<F, S>& rhs);

	virtual ~PairArray();

	void push_back(F& first, S& second);

private:
	int nElements_;
	F* first_;
	S* second_;
};


template <typename F, typename S>
std::ostream& operator<<(std::ostream& out,
	const PairArray<F, S>& thingToPrint)
{
	thingToPrint.print(out);
	return out;
}

template <typename F, typename S>
PairArray<F, S>::PairArray() : nElements_(0),
   first_(0), second_(0)
{
}

template <typename F, typename S>
PairArray<F, S>::PairArray(const PairArray<F, S>& orig) :
	nElements_(orig.nElements_)
{
	first_ = new F[nElements_];
	second_ = new S[nElements_];

	for (int i = 0; i < nElements_; i++)
	{
		first_[i] = orig.first_[i];
		second_[i] = orig.second_[i];
	}
}

template <typename F, typename S>
void PairArray<F, S>::push_back(F& first, S& second)
{
	// Allocate new memory
	F* newFirst = new F[nElements_ + 1];
	S* newSecond = new S[nElements_ + 1];

	// Copy the old memory to the new 
	for (int i = 0; i < nElements; i++)
	{
		newFirst[i] = first_[i];
		newSecond[i] = second_[i];
	}
	
	// Delete the old memory
	delete [] first_;
	delete [] second_;

	// Point to the new member
	first_ = newFirst;
	second_ = newSecond;

	// Add the element to the end
	first_[nElements_] = first;
	second_[nElements_] = second;

	// Increment the number of elements.
	++nElements_;
}

template <typename F, typename S>
PairArray<F, S>& PairArray<F, S>::operator=(const PairArray<F, S>& rhs)
{
    // Check for self-assignment
	if (this != &rhs)
	{
		// Delete the old memory
		delete[] first_;
		delete[] second_;

		// Allocate new memory
		first_ = new F[rhs.nElements_];
		second_ = new S[rhs.nElements_];

		// Copy the elements from the rhs
		for (int i = 0; i < rhs.nElements_; i++)
		{
			first_[i] = rhs.first_[i];
			second_[i] = rhs.second_[i];
		}

		nElements_ = rhs.nElements_;
	}

	return *this;
}

template <typename F, typename S>
void PairArray<F, S>::print(std::ostream& out) const
{
	for (int i = 0; i < nElements_; i++)
	{
		out << first_[i] << endl;
		out << second_[i] << endl;
	}
}

template <typename F, typename S>
PairArray<F, S>::~PairArray()
{
	delete[] first_;
	delete[] second_;
}
